import json
import os

import dash
from dash import html
from dash import dcc
import dash_bootstrap_components as dbc
from dash.dependencies import Output
from dash.dependencies import Input
from dash.dependencies import State
from dash.exceptions import PreventUpdate
from PIL import Image
import contextily as ctx

CURRENT_PATH = os.path.abspath(__file__)

leaflet = "https://cdnjs.cloudflare.com/ajax/libs/leaflet/1.9.4/leaflet.css"
html2canvas = (
    "https://cdnjs.cloudflare.com/ajax/libs/html2canvas/1.4.1/html2canvas.min.js"
)
jQuery = "https://code.jquery.com/jquery-3.6.0.slim.min.js"
app = dash.Dash(
    __name__,
    external_scripts=[jQuery, html2canvas],
    external_stylesheets=[leaflet, dbc.themes.BOOTSTRAP, dbc.themes.GRID],
)

app.index_string = """
<!DOCTYPE html>
<html>
    <head>
        <script>
            L_PREFER_CANVAS=true;
            L_SCROLL_WHEEL_ZOOM=false;
        </script>
        {%metas%}
        <title>{%title%}</title>
        {%favicon%}
        {%css%}
    </head>
    <body>
        {%app_entry%}
        <footer>
            {%config%}
            {%scripts%}
            {%renderer%}
        </footer>
    </body>
</html>
"""


@dash.callback(
    Output("map-matplotlib", "src"),
    [
        Input("download", "n_clicks"),
        State("view-dropdown", "value"),
        State("min-lat", "value"),
        State("max-lat", "value"),
        State("min-lon", "value"),
        State("max-lon", "value"),
    ],
    prevent_initial_call=True,
)
def plot_data(n_clicks, tile_layer, min_lat, max_lat, min_lon, max_lon):
    if (
        min_lat is None
        or max_lat is None
        or min_lon is None
        or max_lon is None
        or tile_layer is None
    ):
        raise PreventUpdate

    print("\nCreating preview figure...")

    # Get image from source and coordinates
    map_layers = json.load(
        open(os.path.abspath(os.path.join(CURRENT_PATH, "../conf/map_layers.json")))
    )
    img, bounds = ctx.bounds2img(
        min_lon,
        min_lat,
        max_lon,
        max_lat,
        ll=True,
        source=map_layers[tile_layer]["url"],
    )

    fname = "map_" + str(min_lat) + "_" + str(max_lat) + "_" + str(min_lon) + "_" + str(max_lon) + ".png"
    
    output_path = os.path.abspath(
        os.path.join(CURRENT_PATH, "../assets/images/", fname)
    )
    map = Image.fromarray(img)
    map.save(output_path)

    msg = "Current limits: \n"
    msg += "Min lat: " + str(min_lat) + "\n"
    msg += "Max lat: " + str(max_lat) + "\n"
    msg += "Min lon: " + str(min_lon) + "\n"
    msg += "Max lon: " + str(max_lon)
    print(msg)

    print(f"Image was downloaded in {output_path}")

    return r"/assets/images/" + r"{}".format(fname)


map_layers = json.load(
    open(os.path.abspath(os.path.join(CURRENT_PATH, "../conf/map_layers.json")))
)
app.layout = html.Div(
    [
        dbc.Row(
            [
                dbc.Col(html.Div("Tile layer", style={"margin": "5px"}), width=2),
                dbc.Col(html.Div("Minimum latitude", style={"margin": "5px"}), width=2),
                dbc.Col(html.Div("Maximum latitude", style={"margin": "5px"}), width=2),
                dbc.Col(
                    html.Div("Minimum longitude", style={"margin": "5px"}), width=2
                ),
                dbc.Col(
                    html.Div("Maximum longitude", style={"margin": "5px"}), width=2
                ),
            ]
        ),
        dbc.Row(
            [
                dbc.Col(
                    dcc.Dropdown(
                        id="view-dropdown",
                        options=[
                            {"label": map_layers[style]["name"], "value": style}
                            for style in map_layers
                        ],
                        value="carto-positron-no-labels",
                        style={"margin": "5px"},
                    ),
                    width=2,
                ),
                dbc.Col(
                    dcc.Input(
                        id="min-lat",
                        type="number",
                        placeholder="Input minimum latitude",
                        style={"margin": "5px", "height": "35px"},
                    ),
                    width=2,
                ),
                dbc.Col(
                    dcc.Input(
                        id="max-lat",
                        type="number",
                        placeholder="Input maximum latitude",
                        style={"margin": "5px", "height": "35px"},
                    ),
                    width=2,
                ),
                dbc.Col(
                    dcc.Input(
                        id="min-lon",
                        type="number",
                        placeholder="Input minimum longitude",
                        style={"margin": "5px", "height": "35px"},
                    ),
                    width=2,
                ),
                dbc.Col(
                    dcc.Input(
                        id="max-lon",
                        type="number",
                        placeholder="Input maximum longitude",
                        style={"margin": "5px", "height": "35px"},
                    ),
                    width=2,
                ),
                dbc.Col(
                    dbc.Button(
                        "Download",
                        id="download",
                        className="me-2",
                        n_clicks=0,
                    ),
                    width=2,
                ),
            ]
        ),
        html.Img(id="map-matplotlib", style={"margin": "5px"}),
        html.Div(id="hidden-div", style={"display": "none"}),
    ]
)

if __name__ == "__main__":
    app.run_server(debug=True, port=7782)
